@@ -2,7 +2,7 @@
  * =========================================================================
  * This file is part of the GrInvIn project - http://www.grinvin.org
  * 
- * Copyright (C) 2005-2007 Universiteit Gent
+ * Copyright (C) 2005-2008 Universiteit Gent
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,20 +34,23 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
 
-import org.grinvin.DefaultEmbedding;
-import org.grinvin.Edge;
-import org.grinvin.Element;
-import org.grinvin.Embedding;
-import org.grinvin.EmbeddingModel;
-import org.grinvin.EmbeddingView;
-import org.grinvin.Graph;
-import org.grinvin.MutableEmbeddingModel;
-import org.grinvin.Vertex;
+import org.grinvin.graphs.Annotation;
+import org.grinvin.graphs.AnnotationModel;
+import org.grinvin.graphs.DefaultEmbedding;
+import org.grinvin.graphs.Edge;
+import org.grinvin.graphs.Element;
+import org.grinvin.graphs.Embedding;
+import org.grinvin.graphs.EmbeddingModel;
+import org.grinvin.graphs.EmbeddingView;
+import org.grinvin.graphs.MutableAnnotationModel;
+import org.grinvin.graphs.MutableEmbeddingModel;
+import org.grinvin.gred.undoable.UndoableGraph;
+import org.grinvin.graphs.Vertex;
 import org.grinvin.gred.undoable.AddElements;
 import org.grinvin.gred.undoable.ChangeCoordinates;
 import org.grinvin.gred.undoable.InternationalizedUndoableChange;
 import org.grinvin.gred.undoable.RemoveElements;
-import org.grinvin.render.Renderer;
+import org.grinvin.graphs.render.Renderer;
 
 /**
  * Graph panel which is backed by a 2-dimensional mutable embedding model.
@@ -80,17 +83,17 @@ public class MutableGraphPanel extends GraphPanel {
      * @param context Graph context for this panel.
      * @param scale Number of pixels corresponding to a unit length in the embedding.
      */
-    public MutableGraphPanel(MutableEmbeddingModel embedding,
+    public MutableGraphPanel(MutableEmbeddingModel embedding, MutableAnnotationModel annotation,
             Renderer renderer, GraphContext context, double scale) {
-        super(embedding, renderer, context, scale);
+        super(embedding, annotation, renderer, context, scale);
         this.snapToGuides = false;
         this.dirty = false;
     }
     
     // overrides GraphPanel
-    @Override public void setEmbedding(EmbeddingModel originalEmbedding) {
+    @Override public void setGraph(EmbeddingModel originalEmbedding, AnnotationModel annotation) {
         if (originalEmbedding instanceof MutableEmbeddingModel) {
-            super.setEmbedding(originalEmbedding);
+            super.setGraph(originalEmbedding, annotation);
         } else
             throw new IllegalArgumentException("Embedding model must be mutable");
     }
@@ -103,8 +106,8 @@ public class MutableGraphPanel extends GraphPanel {
     /**
      * Returns the (mutable) graph displayed by this panel.
      */
-    @Override public Graph getGraph() {
-        return (Graph)this.graph;
+    @Override public UndoableGraph getGraph() {
+        return (UndoableGraph)this.graph;
     }
     
     /**
@@ -114,6 +117,13 @@ public class MutableGraphPanel extends GraphPanel {
         return (Embedding)this.embedding;
     }
     
+    /**
+     * Returns the (mutable) annotation used by this panel.
+     */
+    @Override public Annotation getAnnotation() {
+        return (Annotation)this.annotation;
+    }
+    
     /**
      * Removes the current graph
      */
@@ -162,7 +172,8 @@ public class MutableGraphPanel extends GraphPanel {
      * Snaps to guides if currently enabled.
      */
     private Vertex addNewVertex(final int mouseX, final int mouseY, final Object annotation) {
-        Vertex vertex = getGraph().addNewVertex(annotation);
+        Vertex vertex = getGraph().addNewVertex();
+        getAnnotation().setAnnotation(vertex, annotation);
         double[] coords = new double[] { (mouseX - getWidth()/2)/scale,
         (getHeight()/2 - mouseY)/scale };
         if (snapToGuides)
@@ -207,11 +218,11 @@ public class MutableGraphPanel extends GraphPanel {
         Vertex first = edge.getFirstEndpoint();
         Vertex second = edge.getSecondEndpoint();
         getGraph().remove(edge);
-        elements.add(getGraph().addNewEdge(first, vertex, null));
-        elements.add(getGraph().addNewEdge(vertex, second, null));
+        elements.add(getGraph().addNewEdge(first, vertex));
+        elements.add(getGraph().addNewEdge(vertex, second));
         
         if (joinToAnchor) {
-            elements.add(getGraph().addNewEdge((Vertex)anchor, vertex, null));
+            elements.add(getGraph().addNewEdge((Vertex)anchor, vertex));
         }
         
         // register with undoManager
@@ -416,14 +427,14 @@ public class MutableGraphPanel extends GraphPanel {
         
         coords[0] /= nrOfVerticesInSelection;
         coords[1] /= nrOfVerticesInSelection;
-        Vertex vertex = getGraph().addNewVertex(null);
+        Vertex vertex = getGraph().addNewVertex();
         if (snapToGuides)
             guides.snap(coords);
         getEmbedding().setCoordinates(vertex, coords);
         
         List<Element> edges = new ArrayList<Element>();
         for(Vertex v : adjacentVertices)
-            edges.add(getGraph().addNewEdge(vertex, v, null));
+            edges.add(getGraph().addNewEdge(vertex, v));
         
         // remove selected vertices and their edges
         List<Element> list = new ArrayList<Element>();
@@ -470,7 +481,7 @@ public class MutableGraphPanel extends GraphPanel {
                 vertices[index++] = v2;
             for(Vertex vertex : vertices){
                 if(graph.areAdjacent(v,vertex)){
-                    Vertex newVertex = getGraph().addNewVertex(null);
+                    Vertex newVertex = getGraph().addNewVertex();
                     double[] coords = new double[2];
                     coords[0] = vCoords[0] + (embedding.getCoordinates(vertex)[0]-vCoords[0])/5;
                     coords[1] = vCoords[1] + (embedding.getCoordinates(vertex)[1]-vCoords[1])/5;
@@ -479,7 +490,7 @@ public class MutableGraphPanel extends GraphPanel {
                     getGraph().remove(e);
                     undoManager.add(new RemoveElements(getGraph(), e), false);
                     undoManager.add(new AddElements(getGraph(), newVertex), false);
-                    undoManager.add(new AddElements(getGraph(), getGraph().addNewEdge(newVertex, vertex, null)), false);
+                    undoManager.add(new AddElements(getGraph(), getGraph().addNewEdge(newVertex, vertex)), false);
                 }
             }
             getGraph().remove(v);
@@ -503,7 +514,7 @@ public class MutableGraphPanel extends GraphPanel {
         for(int i=0; i<selectedVertices.size()-1; i++)
             for(int j=i+1; j<selectedVertices.size(); j++)
                 if(!graph.areAdjacent(selectedVertices.get(i),selectedVertices.get(j)))
-                    newEdges.add(getGraph().addNewEdge(selectedVertices.get(i),selectedVertices.get(j),null));
+                    newEdges.add(getGraph().addNewEdge(selectedVertices.get(i),selectedVertices.get(j)));
         
         undoManager.add(new AddElements(getGraph(), newEdges), true);
         setDirty(true);
@@ -597,7 +608,7 @@ public class MutableGraphPanel extends GraphPanel {
                     removedEdges.add(e);
                     getGraph().remove(e);
                 } else {
-                    addedEdges.add(getGraph().addNewEdge(vertices[i],vertices[j], null));
+                    addedEdges.add(getGraph().addNewEdge(vertices[i],vertices[j]));
                 }
             }
         }
@@ -631,7 +642,7 @@ public class MutableGraphPanel extends GraphPanel {
             for(int i=0; i<n-1; i++)
                 for(int j=i+1; j<n; j++)
                     if(!graph.areAdjacent(vertices[i], vertices[j]) && degree[i]+degree[j]>=n){
-                        addedEdges.add(getGraph().addNewEdge(vertices[i], vertices[j], null));
+                        addedEdges.add(getGraph().addNewEdge(vertices[i], vertices[j]));
                         degree[i]++;
                         degree[j]++;
                     }
@@ -659,7 +670,7 @@ public class MutableGraphPanel extends GraphPanel {
         //place new vertices
         for(Edge e : graph.edges()){
             oldEdges.add(index, e);
-            newVertices[index]=getGraph().addNewVertex(null);
+            newVertices[index]=getGraph().addNewVertex();
             addedVertices.add(newVertices[index]);
             double[] coords = new double[2];
             coords[0] = (embedding.getCoordinates(e.getFirstEndpoint())[0] + embedding.getCoordinates(e.getSecondEndpoint())[0])/2;
@@ -672,7 +683,7 @@ public class MutableGraphPanel extends GraphPanel {
         for(int i=0; i<newOrder-1; i++)
             for(int j=i+1; j<newOrder; j++)
                 if(edgesIntersect((Edge)oldEdges.get(i), (Edge)oldEdges.get(j)))
-                    addedEdges.add(getGraph().addNewEdge(newVertices[i], newVertices[j], null));
+                    addedEdges.add(getGraph().addNewEdge(newVertices[i], newVertices[j]));
         
         //remove old graph
         for(Element e : oldEdges)
