@@ -2,7 +2,7 @@
  * =========================================================================
  * This file is part of the GrInvIn project - http://www.grinvin.org
  * 
- * Copyright (C) 2005-2007 Universiteit Gent
+ * Copyright (C) 2005-2008 Universiteit Gent
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,7 +29,6 @@ package org.grinvin.invariants;
 
 import java.io.File;
 import java.io.IOException;
-
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -41,6 +40,7 @@ import java.util.logging.Logger;
 
 import org.grinvin.factories.FactoryException;
 import org.grinvin.factories.FactoryParameterException;
+import org.grinvin.io.IOFormatException;
 import org.grinvin.params.ParameterList;
 import org.grinvin.util.LocalClassLoader;
 import org.grinvin.xml.XMLUtils;
@@ -70,7 +70,7 @@ public class InvariantManager {
     /**
      * Maps an invariant identifier to the corresponding object of type invariant.
      */
-    private final Map<String, Invariant> invariants;
+    private final Map<String, InvariantExpressionNode> invariants;
     
     /**
      * Maps a generic identifier to the corresponding invariant factory.
@@ -108,8 +108,8 @@ public class InvariantManager {
      * @param path classpath relative path for the definition file of
      * this invariant.
      */
-    public Invariant loadInvariant(String id, String path) throws UnknownInvariantException {
-        Invariant inv = invariants.get(id);
+    public InvariantExpressionNode loadInvariant(String id, String path) throws UnknownInvariantException {
+        InvariantExpressionNode inv = invariants.get(id);
         if (inv == null) {
             int pos = id.indexOf('?');
             String newId = id;
@@ -133,7 +133,11 @@ public class InvariantManager {
      * Return the invariant with the given identifier.
      */
     public Invariant getInvariant(String id) throws UnknownInvariantException {
-        Invariant inv = invariants.get(id);
+        return getInvariantExpressionNode(id);
+    }
+    
+    public InvariantExpressionNode getInvariantExpressionNode(String id) throws UnknownInvariantException {
+        InvariantExpressionNode inv = invariants.get(id);
         if (inv == null) {
             // split off parameter values
             int pos = id.indexOf('?');
@@ -153,6 +157,8 @@ public class InvariantManager {
                 } catch (FactoryParameterException ex) {
                     throw new UnknownInvariantException(
                             "Invalid parameters for invariant", id, ex);
+                } catch (IllegalInvariantComputerException ex) {
+                    throw new UnknownInvariantException("Error in invariant computer", id, ex);
                 } catch (FactoryException ex) {
                     throw new UnknownInvariantException(
                             "Invalid invariant", id, ex);
@@ -240,14 +246,14 @@ public class InvariantManager {
     /**
      * Add contents of element as children to the given group.
      */
-    private void load(Element root, InvariantGroup parent, ResourceBundle bundle) {
+    private void load(Element root, InvariantGroup parent, ResourceBundle bundle) throws IOFormatException {
         for (Object o : root.getChildren()) {
             Element element = (Element)o;
             String name = element.getName();
             if ("id".equals(name)) {
                 String id = element.getTextTrim();
                 try {
-                    parent.add(getInvariant(id));
+                    parent.add(getInvariantExpressionNode(id));
                 } catch (UnknownInvariantException ex) {
                     Logger.getLogger(LOGGER_NAME).warning(
                             "Could not load unknown invariant '" + id +
@@ -279,16 +285,18 @@ public class InvariantManager {
                             ".xml";
                     try {
                         if ("id".equals(childName)) {
-                            Invariant invariant = loadInvariant(childId, path);
+                            InvariantExpressionNode invariant = loadInvariant(childId, path);
                             parent.add(invariant);
                         } else if ("factory".equals(childName)) {
                             InvariantFactory factory = loadInvariantFactory(childId, path);
                             parent.add(factory);
+                        } else {
+                            throw new IOFormatException("Unexpected element <" + childName + "> in XML file");
                         }
                     } catch (UnknownInvariantException ex) {
                         Logger.getLogger(LOGGER_NAME).warning(
                                 "Skipping unknown invariant (factory) '" + childId +
-                                "' with path '" + path +"'");
+                                "' with path '" + path +"' due to '" + ex.getMessage() + "'");
                     }
                 }
             }
@@ -306,6 +314,17 @@ public class InvariantManager {
         return icm.getInvariantComputerFor(invariant);
     }
     
+    /**
+     * Return the invariant computers which can be used to compute values for
+     * the given invariant
+     * @param invariant the {@link Invariant} for which you want a computer
+     * @return the <code>List</code> of <code>InvariantComputer</code>s that
+     * can compute the given <code>Invariant</code>.
+     */
+    public List<InvariantComputer> getInvariantComputersFor(Invariant invariant) {
+        return icm.getInvariantComputersFor(invariant);
+    }
+    
     /**
      * Return the computer factory that corresponds to the given invariant factory.
      */
@@ -318,7 +337,7 @@ public class InvariantManager {
         
         try {
             
-            this.invariants = new HashMap<String, Invariant>();
+            this.invariants = new HashMap<String, InvariantExpressionNode>();
             this.factories = new HashMap<String, InvariantFactory>();
             this.ids = new ArrayList<String> ();
             
@@ -339,6 +358,7 @@ public class InvariantManager {
             this.icm = new InvariantComputerManager(this, localClassLoader);
         
         } catch (Exception ex) {
+            Logger.getLogger(LOGGER_NAME).log (Level.SEVERE, "Could not initialize invariant manager", ex);
             throw new RuntimeException(
                     "Could not initialize invariant manager", ex);
         }
@@ -376,9 +396,9 @@ public class InvariantManager {
     }
     
 // same as getInvariantForFactory but does not check the cache first
-    private Invariant getInvariantForFactoryAux(InvariantFactory factory) throws
-            FactoryParameterException {
-        Invariant result = factory.createInvariant();
+    private InvariantExpressionNode getInvariantForFactoryAux(InvariantFactory factory) throws
+            FactoryParameterException, IllegalInvariantComputerException {
+        InvariantExpressionNode result = factory.createInvariant();
         icm.registerComputerFromFactory(factory, result);
         invariants.put(result.getId(), result);
         return result;
@@ -391,10 +411,10 @@ public class InvariantManager {
      * @throws FactoryParameterException when the factory parameters
      * were not set in an appropriate manner before this method was called.
      */
-    public Invariant getInvariantForFactory(InvariantFactory factory) throws
-            FactoryParameterException, FactoryException {
+    public InvariantExpressionNode getInvariantForFactory(InvariantFactory factory) throws
+            FactoryParameterException, FactoryException, IllegalInvariantComputerException {
         String id = factory.getInvariantId();
-        Invariant result = invariants.get(id);
+        InvariantExpressionNode result = invariants.get(id);
         if (result == null)
             result = getInvariantForFactoryAux(factory);
         return result;
